@@ -11,16 +11,21 @@
  DESCRIPTION
  .nf
 
+ /*
+  * Utility library.
+  */
+#include <dict.h>
+
  /*
   * External interface.
   */
-
 typedef struct CFG_PARSER {
     char   *name;
     char   *(*get_str) (const struct CFG_PARSER *, const char *, const char *,
 			        int, int);
     int     (*get_int) (const struct CFG_PARSER *, const char *, int, int, int);
     int     (*get_bool) (const struct CFG_PARSER *, const char *, int);
+    DICT_OWNER owner;
 } CFG_PARSER;
 
 extern CFG_PARSER *cfg_parser_alloc(const char *);
@@ -30,6 +35,8 @@ extern int cfg_get_int(const CFG_PARSER *, const char *, int, int, int);
 extern int cfg_get_bool(const CFG_PARSER *, const char *, int);
 extern CFG_PARSER *cfg_parser_free(CFG_PARSER *);
 
+#define cfg_get_owner(cfg) ((cfg)->owner)
+
 /* LICENSE
 /* .ad
 /* .fi
