@@ -66,7 +66,6 @@
 /* System library. */
 
 #include <sys_defs.h>
-#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 
@@ -209,10 +208,8 @@ void    pcf_edit_main(int mode, int argc, char **argv)
 	    msg_panic("pcf_edit_main: unknown mode %d", mode);
 	}
 	if ((cvalue = htable_find(table, pattern)) != 0) {
-	    if (edit_value && cvalue->value 
-		&& strcmp(edit_value, cvalue->value) != 0)
-		msg_warn("ignoring earlier request: '%s = %s'",
-			 pattern, cvalue->value);
+	    msg_warn("ignoring earlier request: '%s = %s'",
+		     pattern, cvalue->value);
 	    htable_delete(table, pattern, myfree);
 	}
 	cvalue = (struct cvalue *) mymalloc(sizeof(*cvalue));
