@@ -53,7 +53,7 @@
 /* .IP "SEND_ATTR_DATA(const char *name, ssize_t len, const void *value)"
 /*	The arguments are an attribute name, an attribute value
 /*	length, and an attribute value pointer.
-/* .IP "SEND_ATTR_FUNC(ATTR_PRINT_SLAVE_FN, const void *value)"
+/* .IP "SEND_ATTR_FUNC(ATTR_PRINT_CUSTOM_FN, const void *value)"
 /*	The arguments are a function pointer and generic data
 /*	pointer. The caller-specified function returns whatever the
 /*	specified attribute printing function returns.
@@ -153,7 +153,7 @@ int     attr_vprint64(VSTREAM *fp, int flags, va_list ap)
     HTABLE_INFO **ht_info_list;
     HTABLE_INFO **ht;
     ssize_t len_val;
-    ATTR_PRINT_SLAVE_FN print_fn;
+    ATTR_PRINT_CUSTOM_FN print_fn;
     void   *print_arg;
 
     /*
@@ -211,7 +211,7 @@ int     attr_vprint64(VSTREAM *fp, int flags, va_list ap)
 			 attr_name, (long) len_val);
 	    break;
 	case ATTR_TYPE_FUNC:
-	    print_fn = va_arg(ap, ATTR_PRINT_SLAVE_FN);
+	    print_fn = va_arg(ap, ATTR_PRINT_CUSTOM_FN);
 	    print_arg = va_arg(ap, void *);
 	    print_fn(attr_print64, fp, flags | ATTR_FLAG_MORE, print_arg);
 	    break;
@@ -269,6 +269,7 @@ int     main(int unused_argc, char **argv)
     htable_enter(table, "foo-name", mystrdup("foo-value"));
     htable_enter(table, "bar-name", mystrdup("bar-value"));
     attr_print64(VSTREAM_OUT, ATTR_FLAG_NONE,
+		 SEND_ATTR_STR("protocol", "test"),
 		 SEND_ATTR_INT(ATTR_NAME_INT, 4711),
 		 SEND_ATTR_LONG(ATTR_NAME_LONG, 1234L),
 		 SEND_ATTR_STR(ATTR_NAME_STR, "whoopee"),
@@ -277,11 +278,15 @@ int     main(int unused_argc, char **argv)
 		 SEND_ATTR_LONG(ATTR_NAME_LONG, 4321L),
 		 ATTR_TYPE_END);
     attr_print64(VSTREAM_OUT, ATTR_FLAG_NONE,
+		 SEND_ATTR_STR("protocol", "test"),
 		 SEND_ATTR_INT(ATTR_NAME_INT, 4711),
 		 SEND_ATTR_LONG(ATTR_NAME_LONG, 1234L),
 		 SEND_ATTR_STR(ATTR_NAME_STR, "whoopee"),
 	       SEND_ATTR_DATA(ATTR_NAME_DATA, strlen("whoopee"), "whoopee"),
 		 ATTR_TYPE_END);
+    attr_print64(VSTREAM_OUT, ATTR_FLAG_NONE,
+		 SEND_ATTR_STR("protocol", "not-test"),
+		 ATTR_TYPE_END);
     if (vstream_fflush(VSTREAM_OUT) != 0)
 	msg_fatal("write error: %m");
 
